@@ -28,16 +28,16 @@ pub enum FieldError {
 }
 
 #[derive(Debug, Clone)]
-pub enum RegexOrStr<'b> {
+pub enum RegexOrString {
     Regex(Regex),
-    Str(&'b str),
+    String(String),
 }
 
-impl<'b> RegexOrStr<'b> {
-    fn split(&'b self, line: &'b [u8]) -> Box<dyn Iterator<Item = &'b [u8]> + 'b> {
+impl RegexOrString {
+    fn split<'a>(&'a self, line: &'a [u8]) -> Box<dyn Iterator<Item = &'a [u8]> + 'a> {
         match self {
-            RegexOrStr::Regex(r) => Box::new(r.split(line)),
-            RegexOrStr::Str(s) => Box::new(line.split_str(s)),
+            RegexOrString::Regex(r) => Box::new(r.split(line)),
+            RegexOrString::String(s) => Box::new(line.split_str(s)),
         }
     }
 }
@@ -151,8 +151,9 @@ impl FieldRange {
     pub fn from_header_list(
         list: &[Regex],
         header: &[u8],
-        delim: &RegexOrStr,
+        delim: &RegexOrString,
         header_is_regex: bool,
+        allow_missing: bool,
     ) -> Result<Vec<FieldRange>, FieldError> {
         let mut ranges = vec![];
         let mut found = vec![false; list.len()];
@@ -178,12 +179,14 @@ impl FieldRange {
             }
         }
 
-        if ranges.is_empty() {
-            return Err(FieldError::NoHeadersMatched);
-        }
-        for (i, was_found) in found.into_iter().enumerate() {
-            if !was_found {
-                return Err(FieldError::HeaderNotFound(list[i].as_str().to_owned()));
+        if !allow_missing {
+            if ranges.is_empty() {
+                return Err(FieldError::NoHeadersMatched);
+            }
+            for (i, was_found) in found.into_iter().enumerate() {
+                if !was_found {
+                    return Err(FieldError::HeaderNotFound(list[i].as_str().to_owned()));
+                }
             }
         }
 
@@ -200,7 +203,7 @@ impl FieldRange {
         for i in 0..ranges.len() {
             let j = i + 1;
             if let Some(rng) = ranges.get_mut(i) {
-                rng.pos -= shifted;
+                rng.pos = rng.pos.saturating_sub(shifted);
             }
 
             while j < ranges.len()
@@ -330,6 +333,7 @@ mod test {
         assert_eq!(vec![FieldRange { low: 0, high: 0, pos: 1},  FieldRange { low: 3, high: usize::MAX - 1, pos: 0}, FieldRange { low: 4, high: 7, pos: 2}], FieldRange::from_list("4-,1,5-8").unwrap());
         assert_eq!(vec![FieldRange { low: 0, high: 3, pos: 0}], FieldRange::from_list("-4").unwrap());
         assert_eq!(vec![FieldRange { low: 0, high: 7, pos: 0}], FieldRange::from_list("-4,5-8").unwrap());
+        assert_eq!(vec![FieldRange { low: 0, high: 0, pos: 1 }, FieldRange { low: 2, high: 2, pos: 0}, FieldRange { low: 2, high: 2, pos: 2}], FieldRange::from_list("3,1,3").unwrap());
     }
 
     #[test]
@@ -345,13 +349,14 @@ mod test {
     fn test_parse_header_fields() {
         let header = b"is_cat-isdog-wascow-was_is_apple-12345-!$%*(_)";
         let delim = Regex::new("-").unwrap();
-        let delim = RegexOrStr::Regex(delim);
+        let delim = RegexOrString::Regex(delim);
         let header_fields = vec![
             Regex::new(r"^is_.*$").unwrap(),
             Regex::new("dog").unwrap(),
             Regex::new(r"\$%").unwrap(),
         ];
-        let fields = FieldRange::from_header_list(&header_fields, header, &delim, true).unwrap();
+        let fields =
+            FieldRange::from_header_list(&header_fields, header, &delim, true, false).unwrap();
         assert_eq!(
             vec![
                 FieldRange {
@@ -373,9 +378,10 @@ mod test {
     fn test_parse_header_fields_literal() {
         let header = b"is_cat-is-isdog-wascow-was_is_apple-12345-!$%*(_)";
         let delim = Regex::new("-").unwrap();
-        let delim = RegexOrStr::Regex(delim);
+        let delim = RegexOrString::Regex(delim);
         let header_fields = vec![Regex::new(r"is").unwrap()];
-        let fields = FieldRange::from_header_list(&header_fields, header, &delim, false).unwrap();
+        let fields =
+            FieldRange::from_header_list(&header_fields, header, &delim, false, false).unwrap();
         assert_eq!(
             vec![FieldRange {
                 low: 1,
@@ -390,14 +396,14 @@ mod test {
     fn test_parse_header_fields_literal_header_not_found() {
         let header = b"is_cat-is-isdog-wascow-was_is_apple-12345-!$%*(_)";
         let delim = Regex::new("-").unwrap();
-        let delim = RegexOrStr::Regex(delim);
+        let delim = RegexOrString::Regex(delim);
         let header_fields = vec![
             Regex::new(r"^is_.*$").unwrap(),
             Regex::new("dog").unwrap(),
             Regex::new(r"\$%").unwrap(),
             Regex::new(r"is").unwrap(),
         ];
-        let result = FieldRange::from_header_list(&header_fields, header, &delim, false);
+        let result = FieldRange::from_header_list(&header_fields, header, &delim, false, false);
         assert_eq!(
             result.unwrap_err(),
             FieldError::HeaderNotFound(String::from(r"^is_.*$"))
